function [D] = DMMDonline_estimation(X,Y,co)
%Estimates divergence (D) of X and Y (X(:,t), Y(:,t) is the t^th sample) using the MMD (maximum mean discrepancy) method, online. The number of samples in X [=size(X,2)] and Y [=size(Y,2)] must be equal. Cost parameters are provided in the cost object co.
%
%We make use of the naming convention 'D<name>_estimation', to ease embedding new divergence estimation methods.
%
%REFERENCE: 
%  Arthur Gretton, Karsten M. Borgwardt, Malte J. Rasch, Bernhard Scholkopf and Alexander Smola. A Kernel Two-Sample Test. Journal of Machine  Learning Research 13 (2012) 723-773. See Lemma 14.
%
%Copyright (C) 2012 Zoltan Szabo ("http://nipg.inf.elte.hu/szzoli", "szzoli (at) cs (dot) elte (dot) hu")
%
%This file is part of the ITE (Information Theoretical Estimators) Matlab/Octave toolbox.
%
%ITE is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by
%the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
%
%This software is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
%MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
%
%You should have received a copy of the GNU General Public License along with ITE. If not, see <http://www.gnu.org/licenses/>.

%co.mult:OK.

%verification:
    [dX,num_of_samplesX] = size(X);
    [dY,num_of_samplesY] = size(Y);
    %size(X) must be equal to size(Y):
        if num_of_samplesX~=num_of_samplesY
            disp('Warning: there must be equal number of samples in X and Y. Minimum of the sample numbers has been taken.');
        end
        if dX~=dY
            disp('Error: the dimension of X and Y must be equal.');
        end
    num_of_samples = min(num_of_samplesX,num_of_samplesY);        
    %Number of samples must be even:
        if ~all_even(num_of_samples)
            disp('Warning: the number of samples must be even, the last sample is discarded.');
            num_of_samples = num_of_samples - 1;
        end
        
%initialization: 
    odd_indices = [1:2:num_of_samples];
    even_indices = [2:2:num_of_samples];
    
%Xi,Xj,Yi,Yj:
    Xi = X(:,odd_indices);
    Xj = X(:,even_indices);
    Yi = Y(:,odd_indices);
    Yj = Y(:,even_indices);

D = (K(Xi,Xj,co) + K(Yi,Yj,co) - K(Xi,Yj,co) - K(Xj,Yi,co)) / (num_of_samples/2);

%-----------------------------
function [s] = K(U,V,co)
%Computes \sum_i kernel(U(:,i),V(:,i)), RBF (Gaussian) kernel is used with std=co.sigma

s = sum( exp(-sum((U-V).^2,1)/(2*co.sigma^2)) );
